use actix_web::{get, App, HttpResponse, HttpServer};
use askama::Template;
use clap::Parser;

/// Basic OpenID test client
#[derive(Parser, Debug)]
struct AppConfig {
    /// Listen URL
    #[arg(short, long, env, default_value = "0.0.0.0:7510")]
    listen_addr: String,

    /// Public URL, the URL where this service is accessible, without the trailing slash
    #[arg(short, long, env, default_value = "http://localhost:7510")]
    public_url: String,

    /// URL where the OpenID configuration can be found
    #[arg(short, long, env)]
    configuration_url: String,

    /// OpenID client ID
    #[arg(long, env)]
    client_id: String,

    /// OpenID client secret
    #[arg(long, env)]
    client_secret: String,
}

impl AppConfig {
    pub fn redirect_url(&self) -> String {
        format!("{}/redirect", self.public_url)
    }
}

lazy_static::lazy_static! {
    static ref CONF: AppConfig = {
        AppConfig::parse()
    };
}

#[get("/assets/bootstrap.min.css")]
async fn bootstrap() -> HttpResponse {
    HttpResponse::Ok()
        .content_type("text/css")
        .body(include_str!("../assets/bootstrap.min.css"))
}

#[get("/assets/cover.css")]
async fn cover() -> HttpResponse {
    HttpResponse::Ok()
        .content_type("text/css")
        .body(include_str!("../assets/cover.css"))
}

#[derive(Template)]
#[template(path = "home.html")]
struct HomeTemplate {
    redirect_url: String,
}

#[get("/")]
async fn home() -> HttpResponse {
    HttpResponse::Ok().content_type("text/html").body(
        HomeTemplate {
            redirect_url: CONF.redirect_url(),
        }
        .render()
        .unwrap(),
    )
}

#[actix_web::main]
async fn main() -> std::io::Result<()> {
    env_logger::init_from_env(env_logger::Env::new().default_filter_or("info"));
    log::info!("Will listen on {}", CONF.listen_addr);

    HttpServer::new(|| App::new().service(bootstrap).service(cover).service(home))
        .bind(&CONF.listen_addr)
        .expect("Failed to bind server!")
        .run()
        .await
}

#[cfg(test)]
mod test {
    use crate::AppConfig;

    #[test]
    fn verify_cli() {
        use clap::CommandFactory;
        AppConfig::command().debug_assert();
    }
}
